/**
 * 
  ps2_hobo_nicola.ino Main sketch of "Hobo-nicola PS/2 adapter" using Hobo-nicola keyboard and adapter library.
  Copyright (c) 2018 Takeshi Higasa
  
  This file is part of "Hobo-nicola keyboard and adapter".

  "Hobo-nicola keyboard and adapter" is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  "Hobo-nicola keyboard and adapter" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Hobo-nicola keyboard and adapter".  If not, see <http://www.gnu.org/licenses/>.
  
  hoboNicolaLibrary 1.6.1   Apr. 1. 2022   use hobo_sleep.
  
*/

#include "hobo_nicola.h"
#include "hid_wrap.h"
#include "hobo_sleep.h"
#include "ps2_kbd.h"

ps2_kbd* ps2;

// Function keys with Fn-key pressed.
static const uint16_t fn_keys[] PROGMEM = {
	HID_U_ARROW, FN_MEDIA_VOL_UP,
	HID_D_ARROW, FN_MEDIA_VOL_DOWN,
	HID_R_ARROW, FN_MEDIA_SCAN_NEXT,
	HID_L_ARROW, FN_MEDIA_SCAN_PREV,
	HID_ENTER,	 FN_MEDIA_PLAY_PAUSE,
	HID_DELETE,	 FN_MEDIA_STOP,
	HID_END,		 FN_MEDIA_MUTE,
	HID_ESCAPE | WITH_R_CTRL,	 FN_SYSTEM_SLEEP,		// Ctrl + App + Esc 
	HID_ESCAPE | WITH_R_SHIFT | WITH_R_CTRL,	 FN_SYSTEM_POWER_DOWN,
  HID_S			| WITH_R_CTRL , FN_SETUP_MODE,

	0, 0
};

class ps2_hobo_nicola : public HoboNicola {
public:
	ps2_hobo_nicola() {}
	virtual const uint16_t* get_fn_keys() { return fn_keys; }

	virtual void nicola_led(uint8_t on) { digitalWrite(LED_BUILTIN_RX, on ? 0 : 1); }
	virtual void toggle_nicola_led() { digitalWrite(LED_BUILTIN_RX, digitalRead(LED_BUILTIN_RX) ^ 1); }
	virtual void error_led(uint8_t on) { digitalWrite(LED_BUILTIN_TX, on ? 0 : 1); }

};

ps2_hobo_nicola hobo_nicola;

class ps2_event : public keyboard_notify {
  public:
    virtual void modifier_changed(uint8_t before, uint8_t after) { }
    virtual void raw_key_event(uint8_t key, bool released = false) {
			if (is_usb_suspended()) {
				usb_wakeup();
				ps2->kbd_reset();
				return;
			}
		};
    virtual void key_pressed  (uint8_t key, uint8_t mod) { hobo_nicola.key_event(key, true);  };
    virtual void key_released  (uint8_t key, uint8_t mod) { hobo_nicola.key_event(key, false); };
};

// AVRのみ有効。
#ifdef ARDUINO_ARCH_AVR
#include <avr/sleep.h>
#include <avr/wdt.h>
#endif

// iom32u4.hにPRTIM4が定義されていない
#ifndef PRIM4
#define PRTIM4 4
#endif

void setup() {
  ADCSRA = ADCSRA & 0x7f;
  ACSR = ACSR | 0x80;
  PRR0 = _BV(PRTWI) | _BV(PRTIM1) | _BV(PRSPI) | _BV(PRADC);		
	PRR1 = _BV(PRTIM4) | _BV(PRTIM3) | _BV(PRUSART1);	//

	HoboNicola::init_hobo_nicola(&hobo_nicola);
  ps2 = ps2_kbd::getInstance();

  if (!ps2->begin(new ps2_event, LED_BUILTIN_RX))
    hobo_nicola.error_blink(400);
}


void loop() {
  ps2->task(hobo_nicola.get_hid_led_state());
  hobo_nicola.idle();
	
	if (is_usb_suspended() ) {
		hobo_nicola.error_led(false);
		hobo_nicola.nicola_led(false);
		pinMode(LED_BUILTIN_RX, INPUT_PULLUP);
		pinMode(LED_BUILTIN_TX, INPUT_PULLUP);
		enter_sleep(500);
		pinMode(LED_BUILTIN_RX, OUTPUT);
		pinMode(LED_BUILTIN_TX, OUTPUT);
	} else
		enter_sleep();
}
