/**
 * 
	nicola_state.cpp   NICOLA-state-machine implements simultaneous typing method of "Hobo-nicola keyboard and adapter library".
  Copyright (c) 2018 Takeshi Higasa, okiraku-camera.tokyo
  
  This file is part of "Hobo-nicola keyboard and adapter library".

  "Hobo-nicola keyboard and adapter" is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by the Free Software Foundation, 
  either version 3 of the License, or (at your option) any later version.

  "Hobo-nicola keyboard and adapter" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Hobo-nicola keyboard and adapter".  If not, see <http://www.gnu.org/licenses/>.

    Included in hoboNicola 1.5.0.		July. 31. 2021.
		based on nicola_keyboard.cpp
*/

#include "hobo_nicola.h"
#include "hobo_settings.h"

void HoboNicola::timer_tick(unsigned long now) {
  if (event_time != 0 && (now > event_time)) {
    event_time = 0;
    nicola_state(Time_out, 0);
  }
  if (Settings().is_self_repeat() && (repeat_time != 0) && (now > repeat_time)) {
    nicola_state(Key_repeat, 0);
    repeat_time = now + repeat_interval;
  }
}

void HoboNicola::moji_set(uint16_t param) {
	moji = param;
	moji_time = millis();
	event_time = moji_time + e_charTime;
	repeat_moji = repeat_oyayubi = 0;
	immediate_output(moji);
}

void HoboNicola::oyayubi_set(uint16_t param) {
	oyayubi = param;
	oyayubi_time = millis();
	event_time = oyayubi_time + e_oyaTime;
	repeat_moji = repeat_oyayubi = 0;
}

// 即時出力用の前文字キャンセル。
#define sendBS() stroke(HID_BACKSP, 0, true)	

static bool immediate = false;

void HoboNicola::nicola_state(nicola_event_t e, uint16_t param) {
	unsigned long now = millis();
	if (e == All_off) {
		repeat_time = 0;
	}
	switch(state) {
	case Initial_State:
		event_time = moji_time = oyayubi_time = 0;
		oyayubi = moji = 0;
		repeat_moji = repeat_oyayubi = 0;
		repeat_time = 0;
		immediate = false;
		switch(e) {
		case Moji_pressed:
			moji_set(param);
			state = Character_State;
			break;
		case Oyayubi_pressed:
			oyayubi_set(param);
			state = Oyayubi_State;
			break;
		default:
			break;
		}
		break;
	case Character_State:
		switch(e) {
		case Init:
			state = Initial_State;
			break;
		case All_off:
		case Key_released:
			output();
			state = Initial_State;
			break;
		case Moji_pressed:
			output();
			moji_set(param);
			break;
		case Oyayubi_pressed:
			oyayubi_set(param);
			state = Char_Oya_State;
			break;
		case Time_out:
			repeat_moji = moji;
			output();
			if (Settings().is_single_oyayubi_mode()) {	// 長押し待ちへ
				event_time = now + e_longpressTime;
				state = Release_Wait_State;
			} else {
				state = Repeat_State;
				repeat_time = millis() + repeat_delay;
			}
			break;
		default:
			break;			
		}
		break;

	case Release_Wait_State:	// 長押しタイムアウト待ち
		switch(e) {
		case Moji_pressed:
			moji_set(param);
			state = Character_State;
			break;
		case Oyayubi_pressed:
			oyayubi_set(param);
			state = Oyayubi_State;
			break;
		case Time_out:	// 長押し時間経過
			repeat_oyayubi = NID_LONG_PRESSED;
			output();	
			state = Repeat_State;
			repeat_time = millis() + repeat_delay;
			break;
		default:
			state = Initial_State;
			break;			
		}
		break;
	case Oyayubi_State:
		switch(e) {
		case Init:
			state = Initial_State;
			break;
		case Moji_pressed:
			repeat_oyayubi = oyayubi;
			repeat_moji = param;
			output();
			state = Repeat_State;
			repeat_time = millis() + repeat_delay;
			break;
		case All_off:
			output();
			state = Initial_State;
			break;
		case Key_released:
			if (param == oyayubi) {
				output();
				state = Initial_State;
			}
			break;
		case Oyayubi_pressed:
			output();
			oyayubi_set(param);
			break;
		case Time_out:
			if (!dedicated_oyakeys) {
				repeat_oyayubi = oyayubi;
				output();
				state = Repeat_State;
				repeat_time = millis() + repeat_delay;
			}
			break;
		default:
			break;			
		}
		break;
	case Char_Oya_State:
		switch(e) {
		case Init:
			state = Initial_State;
			break;
		case All_off:
			output();
			state = Initial_State;
			break;
		case Oyayubi_pressed:
			output();
			oyayubi_set(param);
			state = Oyayubi_State;
			break;
		case Moji_pressed:
			if ((oyayubi_time - moji_time) < (now - oyayubi_time)) {
				output();   // 先行の文字と親指
				moji_set(param);
				state = Character_State;
			} else {
				uint16_t oya = oyayubi;
				oyayubi = 0;
				if (!immediate)
					output();   // 先行の文字だけ出す
				immediate = false;
				repeat_oyayubi = oya;
				repeat_moji = param;
				output();   // 後追いの文字と親指
				state = Repeat_State;
				repeat_time = millis() + repeat_delay;
			}
			break;
		case Key_released:  // 文字キーが離されたタイミングに応じて単独打鍵とする。
			if ((moji == param) && (now - oyayubi_time < e_nicolaTime) && (oyayubi_time - moji_time) > (now - oyayubi_time)) {
				uint16_t oya = oyayubi;
				oyayubi = 0;
				if (!immediate)
					output();  
				moji = 0;
				immediate = false;
				oyayubi = oya;
				state = Oyayubi_State;
				break;
			}
			output();
			state = Initial_State;
			break;
		case Time_out:
			repeat_oyayubi = oyayubi;
			repeat_moji = moji;
			output();
			state = Repeat_State;
			repeat_time = millis() + repeat_delay;
			break;
		default:
			break;		
		}
		break;
	case Repeat_State:
		switch(e) {
		case Key_repeat:
			output();
			break;
		case Oyayubi_pressed:
			oyayubi_set(param);
			moji = 0;
			state = Oyayubi_State;
			break;		
		case Moji_pressed:
			moji_set(param);
			state = Character_State;
			break;
		case Time_out:
			break;	// ignore time-out event.	
		default:
			state = Initial_State;
			break;
		}	
		break;
	}
}

#define isShiftPressed() (bool) ((modifiers & HID_SHIFT_MASK) != 0)

void HoboNicola::output() {
	if (repeat_moji || repeat_oyayubi) {	// 長押しのときもrepeat_mojiに入っている。
		moji = repeat_moji;
		oyayubi = repeat_oyayubi;
	}
	if (moji == 0 && oyayubi) {
		stroke(HIGHBYTE(oyayubi), modifiers);
		oyayubi = 0;
		return;
	}
	if (immediate) {
		immediate = false;
		if (oyayubi)
			sendBS();
		else
			return;
	}
	// セットアップモードに入るかどうか。
	if (LOWBYTE(moji) == NID_SETUP_KEY) {
		if ((LOWBYTE(oyayubi) == NID_LEFT_OYAYUBI) || (LOWBYTE(oyayubi) == NID_RIGHT_OYAYUBI)) {
			oyayubi = 0;
			if (setup_mode)
				setup_mode = false;
			else
				setup_mode = true;
		} else if (oyayubi == 0)
			setup_mode = false;
		moji = 0;
		return;
	}
	if (setup_mode) {
		setup_options(LOWBYTE(moji));
		moji = 0;
		oyayubi = 0;
		return;
	}

	if (isShiftPressed() && oyayubi == 0)
		oyayubi = NID_SHIFT_KEY;

	const uint8_t* p = get_output_data(moji, oyayubi);
	oyayubi = 0;
	moji = 0;
	if (p) {
		if (state == Release_Wait_State) {
			sendBS();	// タイムアウトで出した文字をキャンセル。
			delay(2);
		}
//		send_PGM_string2(p);
		send_PGM_string(p);
	}
}

void HoboNicola::immediate_output(uint16_t moji) {
	if (!Settings().is_immediate_output() || setup_mode || LOWBYTE(moji) == NID_SETUP_KEY)
		return;
	const uint8_t* p = get_output_data(moji, isShiftPressed() ? NID_SHIFT_KEY : 0);
	if (p) {
//		send_PGM_string2(p);
		send_PGM_string(p);
		immediate = true;
	}
}

//
// 日本語キーボード用
// HIDキーボードの入力コードから内部コードを得る。
// hid usage id(hid) --> internal code(nid)
//	1 2 3 4 5 6 7 8 9 0 - ^ \... (行末が\ではいけません） 
//  q w e r t y u i o p @ [
//  a s d f g h j k l ; : ]
//  z x c v b n m , . / _

//static static const uint8_t PROGMEM hid_to_nid[] = {
static const uint8_t hid_to_nid[] = {
  0x1e, 0x1f, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x2d, 0x2e, 0x89,	// E段 1～0x0d
  0x14, 0x1a, 0x08, 0x15, 0x17, 0x1c, 0x18, 0x0c, 0x12, 0x13, 0x2f, 0x30,		// D段 0x0e～0x19
  0x04, 0x16, 0x07, 0x09, 0x0a, 0x0b, 0x0d, 0x0e, 0x0f, 0x33, 0x34, 0x32,		// C段 0x1a～0x25
  0x1d, 0x1b, 0x06, 0x19, 0x05, 0x11, 0x10, 0x36, 0x37, 0x38, 0x87				// B段 0x26～0x30
};

static const uint8_t NICOLA_KEYS_COUNT = sizeof(hid_to_nid) ;

uint16_t HoboNicola::get_nid(uint8_t& k) {
	uint16_t m = 0;
	switch(k) {
	case HID_J_COLON:         // C11 : は後退キーに
		k =  HID_BACKSP;
		break;
	case HID_J_RBR_32:	       // C12 [ は取消キーに
		k = HID_ESCAPE;
		break;
	case HID_SPACE:
		if (!dedicated_oyakeys) {
			if (Settings().is_single_oyayubi_mode())	// シングル親指のとき、空白キーは右親指キーとみなす。
				m = MKWORD(k, NID_RIGHT_OYAYUBI);
			if (Settings().is_spc_to_left())
				m = MKWORD(k, NID_LEFT_OYAYUBI);
			else if (Settings().is_spc_to_right())
				m = MKWORD(k, NID_RIGHT_OYAYUBI);
		}
		break;
	case HID_MUHENKAN:
		if (!dedicated_oyakeys)
			m = MKWORD(k, NID_LEFT_OYAYUBI);
		break;
	case HID_HENKAN:
		if (!dedicated_oyakeys)
			m = MKWORD(k, NID_RIGHT_OYAYUBI);
		break;
	case HID_PAUSE:
	case HID_DELETE:
		if (!is_initial_state())
			m = MKWORD(k, NID_SETUP_KEY);	// Oyayubi_Stateのみだろう。
		break;			
	default:
		if (dedicated_oyakeys) {	// 専用の親指キーがある
			if (k == left_oyayubi_code)
				return NID_LEFT_OYAYUBI;
			else if (k == right_oyayubi_code)
				return NID_RIGHT_OYAYUBI;
		}
		for(uint8_t i = 0; i < NICOLA_KEYS_COUNT	; i++) {
			// if (pgm_read_byte(&hid_to_nid[i] == k)
			if (hid_to_nid[i] == k)
				return MKWORD(k, (i + 1));
		}
	}
	return m;  
}
// 出力文字列テーブル
// 同時打鍵ステートマシンの出力(内部コード)をホストに送信するためのローマ字文字列とする。
// hid_to_nid[]と同じ要素構成であること
#include "output_string.h"

#define s_nul	empty_str

static const uint8_t* const output_single[] PROGMEM = {	// 単独打鍵
//1       2        3       4         5         6         7         8         9      10       11       12      13
	s_1,    s_2,     s_3,    s_4,      s_5,      s_6,      s_7,      s_8,      s_9,   s_0,     s_minus, s_pipe, s_bslas,
	s_peri, s_ka,    s_ta,   s_ko,     s_sa,     s_ra,     s_ti,     s_ku,     s_tu,  s_comm,  s_comm,  s_colon,
	s_u,    s_si,    s_te,   s_ke,     s_se,     s_ha,     s_to,     s_ki,     s_i,   s_nn,    s_nul,   s_nul,
	s_peri, s_hi,    s_su,   s_fu,     s_he,     s_me,     s_so,     s_ne,     s_ho,  s_slash, s_uline
};

//
static const uint8_t* const output_left[] PROGMEM = {		// 左親指同時
//1        2        3       4         5         6         7         8         9         10         11         12      13
	s_quest, s_slash, s_tild, s_l_brac, s_r_brac, s_l_brac, s_r_brac, s_l_pren, s_r_pren, s_l_cbrac, s_r_cbrac, s_bslas, s_pipe,
	s_xa, 	 s_e,     s_ri,   s_xya,    s_re,     s_pa,     s_di,     s_gu,     s_du,     s_pi,      s_at,      s_aste,
	s_wo,    s_a,     s_na,   s_xyu,    s_mo,     s_ba,     s_do,     s_gi,     s_po,     s_nul,     s_nul,     s_nul,
	s_xu,    s_minus, s_ro,   s_ya,     s_xi,     s_pu,     s_zo,     s_pe,     s_bo,     s_nul,     s_nul
};

static const uint8_t* const output_right[] PROGMEM = {		// 右親指同時
//1        2        3       4         5         6         7         8         9         10         11         12      13
	s_quest, s_slash, s_tild, s_l_brac, s_r_brac, s_l_brac, s_r_brac, s_l_pren, s_r_pren, s_l_cbrac, s_r_cbrac, s_bslas, s_pipe,
	s_nul,   s_ga,    s_da,   s_go,     s_za,     s_yo,     s_ni,     s_ru,     s_ma,     s_xe,      s_at,      s_aste,
	s_vu,    s_ji,    s_de,   s_ge,     s_ze,     s_mi,     s_o,      s_no,     s_xyo,    s_xtu,     s_nul,     s_nul, 
	s_nul,   s_bi,    s_zu,   s_bu,     s_be,     s_nu,     s_yu,     s_mu,     s_wa,     s_xo,      s_nul
};


static const uint8_t* const output_shift[] PROGMEM = {		// Shiftキー押下中
//1        2        3         4         5         6         7         8         9         10         11         12      13
	s_excl, s_dquot,  s_hash,   s_doll,   s_perct,  s_amp,   s_quot,   s_less,    s_gt,     s_nul,     s_equ,     s_tild, s_pipe,
	s_nul,  s_nul,    s_nul,    s_nul,    s_nul,    s_nul,   s_nul,    s_nul,     s_nul,    s_nul,     s_acgra,   s_aste,
	s_nul,  s_nul,    s_larrow, s_darrow, s_nul,    s_pa,    s_uarrow, s_rarrow,  s_nul,    s_plus,    s_nul,     s_nul,
	s_nul,  s_pi,     s_nul,    s_pu,     s_pe,     s_delete,s_enter,  s_nul,     s_po,     s_quest,   s_uline
};

// シングル親指モード。（右親指同時）
static const uint8_t* const output_right_sigle[] PROGMEM = {		
//1        2        3         4         5         6         7         8         9         10         11         12      13
	s_quest, s_slash, s_tild,   s_l_brac, s_r_brac, s_l_brac, s_r_brac, s_l_pren, s_r_pren, s_l_cbrac, s_r_cbrac, s_bslas, s_pipe,
	s_xa,    s_e,     s_ri,     s_xya,    s_re,     s_yo,     s_ni,     s_ru,     s_ma,     s_xe,      s_at,      s_aste,
	s_wo,    s_a,     s_na,     s_xyu,    s_mo,     s_mi,     s_o,      s_no,     s_xyo,    s_xtu,     s_nul,     s_nul,
	s_xu,    s_minus, s_ro,     s_ya,     s_xi,     s_nu,     s_yu,     s_mu,     s_wa,     s_xo,      s_nul
};

// シングル親指モード 単独打鍵（長押し）
// 濁音にならないキーは何も出力しないことにする。
static const uint8_t* const output_single_long[] PROGMEM = {	
//1        2        3         4         5         6         7         8         9         10         11         12      13
	s_quest, s_slash, s_tild,   s_l_brac, s_r_brac, s_l_brac, s_r_brac, s_l_pren, s_r_pren, s_l_cbrac, s_r_cbrac, s_bslas, s_pipe,
	s_nul,   s_ga,    s_da,     s_go,     s_za,     s_nul,    s_di,     s_gu,     s_du,     s_nul,     s_nul,     s_nul,
	s_vu,    s_ji,    s_de,     s_ge,     s_ze,     s_ba,     s_do,     s_gi,     s_nul,    s_nul,     s_nul,     s_nul,
	s_nul,   s_bi,    s_zu,     s_bu,     s_be,     s_nul,    s_zo,     s_nul,    s_bo,     s_nul,     s_nul
};

// output support
// 出力すべき文字列のアドレスを返す。ATmegaならば文字列はPGM内にある。
// 
const uint8_t*  HoboNicola::get_output_data(uint16_t moji, uint16_t oyayubi) {
	uint8_t n = LOWBYTE(moji) ;
	if (n < 1 || n > NICOLA_KEYS_COUNT)
		return 0;
	n--;
	uint8_t* address = 0;
	if (oyayubi == 0)
		address = (uint8_t*)PGM_READ_ADDR(output_single[n]);
	else if (LOWBYTE(oyayubi) == NID_LEFT_OYAYUBI)
		address = (uint8_t*)PGM_READ_ADDR(output_left[n]);
	else if (LOWBYTE(oyayubi) == NID_RIGHT_OYAYUBI)
		if (Settings().is_single_oyayubi_mode())
			address = (uint8_t*)PGM_READ_ADDR(output_right_sigle[n]);
		else
			address = (uint8_t*)PGM_READ_ADDR(output_right[n]);
	else if (LOWBYTE(oyayubi) == NID_SHIFT_KEY)
		address = (uint8_t*)PGM_READ_ADDR(output_shift[n]);
	else if (LOWBYTE(oyayubi) == NID_LONG_PRESSED)
		address = (uint8_t*)PGM_READ_ADDR(output_single_long[n]);
			
	if (address && PGM_READ_BYTE(address) == 0)
		return 0;
	return (const uint8_t*)address;    
}
